#!/bin/bash

# CC BY-NC-SA 4.0 Felix Baumann
# https://creativecommons.org/licenses/by-nc-sa/4.0
# 2017
# Script is provided as supplemental material
# for the article F. W. Baumann and D. Roller "Overview of German Additive Manufacturing Companies"

# This script parses a CSV list of 3D printing companies (in Germany)
# and utilises a number of helper scripts to 
# extract additional information from web sources

LANG=de_DE.UTF-8

INP=[SPECIFY BASE DIR HERE]/list-3d-printing-companies.txt
OUT_DIR=[SPECIFY BASE DIR HERE]/Internet_Data

OUT=${OUT_DIR}/enhanced_data.csv
WORDLIST_DIR=/mnt/home-disk/experiments/website2wordlist

if [ ! -d ${OUT_DIR}/reg-ger ]; then
	mkdir -p ${OUT_DIR}/reg-ger
fi

SCRIPT_DIR=~/scripts

WLW_SEARCH_SCRIPT=${SCRIPT_DIR}/wlw-search.sh
BING_INFO_SCRIPT=${SCRIPT_DIR}/bing-info.sh
ARCHIVE_INFO_SCRIPT=${SCRIPT_DIR}/archive-info.sh
REG_GER_INFO_SCRIPT=${SCRIPT_DIR}/reg-ger-search.sh
COMMON_WORDS_SCRIPT=${SCRIPT_DIR}/most-common-words.sh

# Clearing ${OUT}
echo -n "" > ${OUT}
echo -n "" > ${OUT}.reg
echo -n "" > ${OUT}.wlw
echo -n "" > ${OUT}.words
echo -n "" > ${OUT}.archive

# Forces the charset of the output files to be UTF-8
iconv -f ascii -t utf-8  ${OUT} -o ${OUT}
iconv -f ascii -t utf-8  ${OUT}.wlw -o ${OUT}.wlw
iconv -f ascii -t utf-8  ${OUT}.words -o ${OUT}.words
iconv -f ascii -t utf-8  ${OUT}.reg -o ${OUT}.reg
iconv -f ascii -t utf-8  ${OUT}.archive -o ${OUT}.archive

# The reg-ger-search.sh script places some temporary
# files in /tmp and names them pub_*.txt, these files
# should be removed beforehand as to not confuse the
# script with pre-existing output
rm -rf /tmp/pub_*.txt

while read l; do
	# The original list can contain empty lines and lines
	# starting with a "-", these lines should be ignored
	if [ "${l}" == "" ]; then
		continue
	elif [ "${l:0:1}" == "-" ]; then
		continue
	fi

	# Parse information from the original file
	COMP_NAME=$( echo "${l}" | awk -F";" '{ print $4 }' )
	COMP_URL=$( echo "${l}" | awk -F";" '{ print $3 }' | awk -F"/" '{ print $3 }' )
	PLZ=$(  echo "${l}" | awk -F";" '{ print $6 }' | awk '{ print $1 }' )
	CITY=$(  echo "${l}" | awk -F";" '{ print $6 }' | awk '{ print $2 }' )

	# Information of the current execution

	BING_INFO=$( ${BING_INFO_SCRIPT} "${COMP_URL}" )
	WLW_INFO=$( ${WLW_SEARCH_SCRIPT} "${COMP_NAME}" 2>/dev/null )
	ARCHIVE_INFO=$( ${ARCHIVE_INFO_SCRIPT} "${COMP_URL}" )

	REG_GER_INFO=$( ${REG_GER_INFO_SCRIPT} "${COMP_NAME}" )
	REG_GER_OUT=$( echo "${REG_GER_INFO}" | head -1l )

	RDIR=""
	RDIR=$( echo "${REG_GER_INFO}" | grep "TDIR" | awk -F"\t" '{ print $2 }' | grep -v "longitude" )

	if [ -f ${WORDLIST_DIR}/${COMP_URL}_accumulated.txt ]; then
		MOST_COMMON_WORDS_INFO=$( timeout 3m ${COMMON_WORDS_SCRIPT} ${WORDLIST_DIR}/${COMP_URL}_accumulated.txt )
	else
		echo "No wordlist found for ${COMP_NAME} and ${COMP_URL}" > /dev/stderr
	fi

	HASH=$( echo "${COMP_NAME}" | md5sum | awk '{ print $1 }' )

	mkdir -p ${OUT_DIR}/reg-ger/${HASH}
	if [ -d ${RDIR} ]; then
		mv ${RDIR}/* ${OUT_DIR}/reg-ger/${HASH}/
	fi

	ADDRESS=$( echo "${REG_GER_OUT}" | awk -F";" '{ print $8 }' )
	if [ "${ADDRESS}" != "" ]; then
		DETAIL_ADDRESS=$( echo "${ADDRESS}" | head -1l | awk -F"," '{ print $2", "$3 }' | tr -d "\"" )
		GEO_INFO=$( timeout 1m wget -O - -q --post-data "scantext=${DETAIL_ADDRESS}&geomode=strictmode&geoit=CSV&region=DE&ok=Next" https://geocode.xyz | tail -1l | sed -e "s/,/;/g" | tr -d '"' | recode html...utf8 | grep -v "longitude" )
		if [ "${GEO_INFO}" == "" ]; then
			GEO_INFO=$( timeout 1m wget -O - -q --post-data "scantext=${PLZ} ${CITY}&geomode=strictmode&geoit=CSV&region=DE&ok=Next" https://geocode.xyz | tail -1l | sed -e "s/,/;/g" | tr -d '"' | recode html...utf8 | grep -v "longitude" )
		fi
	else
		GEO_INFO=$( timeout 1m wget -O - -q --post-data "scantext=${PLZ} ${CITY}&geomode=strictmode&geoit=CSV&region=DE&ok=Next" https://geocode.xyz | tail -1l | sed -e "s/,/;/g" | tr -d '"' | recode html...utf8 | grep -v "longitude" )
	fi

	if [ "${GEO_INFO}" == "" ]; then
		GEO_INFO="N/A;N/A;N/A;N/A;N/A"
	fi

	if [ $( echo "${WLW_INFO}" | grep -m 1 -c "${PLZ}" ) -eq 0 ] && [ $( echo "${WLW_INFO}" | grep -m 1 -c "${CITY}" ) -eq 0 ]; then
		WLW_INFO_BLANK="N/A;N/A;N/A;N/A;N/A;N/A;N/A;N/A;N/A;N/A;N/A;N/A"
		echo "${HASH};${l};${BING_INFO};${ARCHIVE_INFO};${WLW_INFO_BLANK};${REG_GER_OUT};${GEO_INFO};${MOST_COMMON_WORDS_INFO}" >> ${OUT}
	else
		echo "${HASH};${l};${BING_INFO};${ARCHIVE_INFO};${WLW_INFO};${REG_GER_OUT};${GEO_INFO};${MOST_COMMON_WORDS_INFO}" >> ${OUT}
		FT=$( file -b -i ${OUT} | awk '{ print $2 }' )
		if [ "${FT}" == "charset=unknown-8bit" ]; then
			echo -e ":::\tChanged Charset" > /dev/stderr
			tail -1l ${OUT} > /dev/stderr
		fi
	fi

done < ${INP}
