#!/bin/bash

# CC BY-NC-SA 4.0 Felix Baumann
# https://creativecommons.org/licenses/by-nc-sa/4.0
# 2017
# Script is provided as supplemental material
# for the article F. W. Baumann and D. Roller "Overview of German Additive Manufacturing Companies"

# This script downloads all directly linked documents from
# the specified website that are on the same domain and download
# them locally, then transforms these documents to plain text
# and aggregates them in a single file. For this aggregated
# file a wordcloud is generated, via the wordcloud_cli.py script from
# http://github.com/amueller/word_cloud

TMP_D=$( mktemp -d )
PWD=$( pwd )

BASE_URL=${1}
START_T=$( date +"%s.%N" )

BASE=$( echo "${BASE_URL}" | awk -F/ '{print $3}' )

echo "Checking ${BASE}" > /dev/stderr
EXP_DIR=[SPECIFY BASE DIR HERE]/website2wordlist

if [ ! -d ${EXP_DIR} ]; then
	mkdir -p ${EXP_DIR}
fi

cd ${TMP_D}
echo "Storing to ${TMP_D}"  > /dev/stderr
T=$( mktemp -p ${TMP_D} )
DOMAIN=${BASE_URL}

D=$( echo "${DOMAIN}" | awk -F"/" '{ print $3 }' )
if [ "${D}" == "" ]; then
	D=$( echo "${DOMAIN}" | awk -F"/" '{ print $1 }' )
fi

BASE_DOMAIN=$( echo "${D}" | sed -e "s/www\.//g" )

LINKS=$( lynx -accept_all_cookies -dump -listonly "${D:-www.example.org}" 2>/dev/null | grep "http" | awk '{ print $2 }' | sort | uniq | grep "${BASE_DOMAIN:-BROKENNOTAVAIL}" )
if [ "${LINKS}" == "" ]; then
	LINKS=$( elinks -dump "${D:-www.example.org}" | grep "^[ ]\{2,3\}[0-9]\{1,3\}\. " | grep "http" | awk '{ print $2 }' | sort | uniq | grep "${BASE_DOMAIN:-BROKENNOTAVAIL}" )
fi

for l in ${LINKS}; do
	wget --directory-prefix=${TMP_D} -q -nd "${l}" 2>/dev/null
done

ALREADY_SEEN_FILES=""

for i in $( ls ${TMP_D}/* | grep -v "${T}" | grep -v "\.js" ); do
	FTYPE=$( file -i ${i} )
	# Omit files that are not html files, such as JavaScript, Images etc.
	if [ $( grep -c -m 1 "text/html" <<< "${FTYPE}" ) -eq 1 ]; then
		FILE_HASH=$( md5sum ${i} | awk '{ print $1 }' )
		# Omit files that are identical to already processed files
		if [ $( grep -c -m 1 "${FILE_HASH}" <<< ${ALREADY_SEEN_FILES} ) -eq 0 ]; then
			ALREADY_SEEN_FILES="${ALREADY_SEEN_FILES} ${FILE_HASH}"
			elinks -no-references -no-numbering -dump ${i} | sed -e "s/\[IMG\]//g;s/Link: .*$//g" >> ${T}
		else
			echo "Hash of file ${i} already checked, skipping" > /dev/stderr
		fi
	else
		echo "Filetype of file ${i} is ${FTYPE}, skipping" > /dev/stderr
	fi
done

if [ -f ${T} ]; then
	cat ${T} | tr " " "\n" | grep -v "^$" | tr "[A-Z]" "[a-z]" | egrep -v "^(• |\&|\_|\-|═|\[)" | tr "\n" " " > ${T}.words
fi

WIDTH=1024
wordcloud_cli.py --stopwords ${EXP_DIR}/stoplist.txt --background white --height ${WIDTH} --width ${WIDTH} --font /usr/share/cura/resources/themes/cura/fonts/Roboto-Light.ttf --color blue --relative_scaling 0 --text ${T}.words --imagefile /tmp/${BASE}.png

END_T=$( date +"%s.%N" )
DUR=$( awk '{ print $1 - $2 }' <<< "${END_T} ${START_T}" )


cp ${T}.words ${EXP_DIR}/${BASE}_words.txt
mv /tmp/wordcloud.svg ${EXP_DIR}/${BASE}_wordcloud.svg
cat ${T}.words | tr " " "\n" > ${EXP_DIR}/${BASE}_words_single.txt

cp ${T} ${EXP_DIR}/${BASE}_accumulated.txt
mv /tmp/${BASE}.png ${EXP_DIR}/${BASE}_wordcloud.png

echo "Duration for ${BASE} -> ${DUR} s" | tee -a ${EXP_DIR}/${BASE}.dur

cd ${PWD}
rm -rf ${TMP_D}
